#! /usr/bin/perl -w
#---------------------------------------------------------------
# Project         : Linux-Mandrake
# Module          : bin
# File            : accounting.pl
# Author          : Camille Bgnis
# Created On      : Mon Nov 4 2002
# Copyright       : Camille Bgnis <camille@mandrakesoft.com>,
#                    MandrakeSoft under the GPL license
#---------------------------------------------------------------

# synopsis: accounting.pl master-revisions-todo.xml

# This script takes a master revisions document and outputs:

# 1) master-report.html that summarize all modules current status

# 2) costs.html: for each participating author the billing information
# according to its work on this document on a per module basis.


use XML::LibXML;

my $file = shift || die "$0: What's the file to parse?!\n";

# tasks relative cost

my $parser=XML::LibXML->new();
$parser->expand_entities(0);
$parser->load_ext_dtd(0);

# Parse revisions file
my $tree = $parser->parse_file($file);
my $root = $tree->getDocumentElement;

# price per written character
my $price=$root->findvalue('(//revisions/cost)[1]');
# print STDERR "$price\n";


# get all modules IDs
my @allmodules=$root->findnodes('//module');
my @modules;

# Document current major release
my $release=$root->findvalue('(//revnumber)[1]');
# get all finished tasks
my @tasks=$root->findnodes("//revision[not(date='YYYY-MM-DD')]");

# Document Title
my $title=$root->findvalue('/modules/@id');

# get only uniq IDs (module names)
foreach (sort @allmodules) {
    my $id=$_->getAttribute('id');
#   print "$id\n";
    $modules[$#modules+1]=$id unless (($#modules > -1) and ($id eq $modules[$#modules]));
}

my $grandtotal=0;

# create a hash that will associate to each task its weight
my %tasksweight;

my @lists=$root->findnodes('//revisions[@lang]');
foreach my $list (@lists) {
    my $lang=$list->getAttribute('lang');
    my @tasks=$root->findnodes('//revisions[@lang="'.$lang.'"]/*/type');
    foreach my $task (@tasks) {
	my $taskname=$task->findvalue('name');
	my $weight=$task->findvalue('weight') || 1;
	$tasksweight{$taskname.".".$lang}=$weight unless ( $tasksweight{$taskname.".".$lang} );
#	print STDERR "lang=$lang\ttaskname=$taskname\tweight=$weight\ttasksweight{$taskname.$lang}=".$tasksweight{$taskname.".".$lang}."\n";
    }
}


# create a hash that will associate to each language their total cost
my %modulestotalcosts;

# get all tasks
my @alltasks=$root->findnodes("//revision");
foreach my $task (@alltasks) {
    my $chars=$task->parentNode->getAttribute('vendor') || 1;
    my $lang=$task->parentNode->getAttribute('lang');
    my $taskname=$task->findvalue('revremark');
    my $weight=$tasksweight{$taskname.".".$lang} || 0;
    my $total=$weight*$chars;    
    $modulestotalcosts{$lang}+=$total;
#    print STDERR "lang=$lang\ttaskname=$taskname\tchars=$chars\tweight=$weight\ttotal=$total\ttotal=$modulestotalcosts{$lang}\n";
}

# create hash that associates to each language their current cost
my %modulescosts;
# create a hash that associates to each author their contribution
my %authorscosts;
# create a hash that associates to each author their contribution per module
my %authorsmodulescosts;

foreach my $task (@tasks) {
    my $initial=$task->findvalue('authorinitials');
    my $date=$task->findvalue('date');
    my $taskname=$task->findvalue('revremark');
    my $taskrelease=$task->findvalue('revnumber');
    my $chars=$task->parentNode->getAttribute('vendor') || 1;
    my $lang=$task->parentNode->getAttribute('lang');
    my $weight=$tasksweight{$taskname.".".$lang};
    my $ratio=1;
# the revision atribute has a different meaning if used along with remap.... Yes I know that sux.
    if ( ! $task->getAttribute('remap') ) {
	$ratio=$task->getAttribute('revision') || 1;
    }
    my $module=$task->parentNode->parentNode->getAttribute('id');
#	print STDERR "$initial\t$task\t$chars\n";
    my $total=$weight*$ratio*$chars;
# For calculating completion status I don't use change ratio
    $modulescosts{$lang}+=$weight*$chars;
# I am only interested in tasks done for this release to calculate contributions
    if ( $taskrelease eq $release ) {
	$authorscosts{$initial}+=$total;
	$authorsmodulescosts{$module.$initial}+=$total;
    }
    $grandtotal+=$total;

#    if ( $initial eq "cb" ) {
#	print STDERR "\nchars=$chars; weight=$weight; ratio=$ratio; total=$total; remap=".$task->getAttribute('remap')."\n";
#	print STDERR "$module.$taskname.$initial:\t$authorsmodulescosts{$module.$initial}\n";
#    }
}

$grandtotal=sprintf("%.0f",$grandtotal/10*$price);

# inserts per language totals in master-report.html

# Parse html file
$file = "master-report.html";
$tree = $parser->parse_file($file);
my $root_report = $tree->getDocumentElement;

my @totalcells=$root_report->findnodes('//th[@role="total"]');


foreach my $cell (@totalcells) {
    my $lang=$cell->getAttribute('lang');
    $modulescosts{$lang}=0 unless $modulescosts{$lang};
    $modulestotalcosts{$lang}=1 unless $modulestotalcosts{$lang};
    my $percent=sprintf("%.1f",$modulescosts{$lang}/$modulestotalcosts{$lang}*100);
#    print STDERR "\nmodulescosts{$lang}=$modulescosts{$lang}\tmodulestotalcosts{$lang}=$modulestotalcosts{$lang}\tpercent=$percent\n";
    my $text=XML::LibXML::Text->new($percent);
    $cell->appendChild($text);    
}
# write final report
$tree->toFile($file,1);

# Create HTML output for authors contributions

my $doc = XML::LibXML::Document->new();
my $html = $doc->createElement('html');
$doc->setDocumentElement($html);
my $body = $doc->createElement('body');
$html->appendChild($body);

my $cell = $doc->createElement('h1');
my $text = XML::LibXML::Text->new($title);
$cell->appendChild($text);
$body->appendChild($cell);
$cell = $doc->createElement('h2');
$text = XML::LibXML::Text->new("Release: $release");
$cell->appendChild($text);
$body->appendChild($cell);
$cell = $doc->createElement('h3');
$text = XML::LibXML::Text->new("Grand total: $grandtotal");
$cell->appendChild($text);
$body->appendChild($cell);

my $table = $doc->createElement('table');
$table->setAttribute('border','1');
$body->appendChild($table);

# print table header
my $authorsline = $doc->createElement('tr');
$table->appendChild($authorsline);
$cell = $doc->createElement('td');
$text = XML::LibXML::Text->new("Module");
$cell->appendChild($text);
$authorsline->appendChild($cell);

foreach my $author (sort keys %authorscosts) {
    my $cell = $doc->createElement('td');
    $cell->setAttribute('role','author');
    $cell->setAttribute('title',$author);
#    my $name=$root->findvalue('//authorgroup/*[@id=$author]/*');
    my @name=$root->findnodes("//authorgroup/*[\@id=\"$author\"]/*/text()");
    my $name="";
    foreach my $name_part (@name) {
	$name .= $name_part->nodeValue()." ";
    }
    my $text = XML::LibXML::Text->new($name);
    $cell->appendChild($text);
    $authorsline->appendChild($cell);
#    print "$modulefile\n";
    }

foreach my $module (sort @modules) {
    my $cost = $doc->createElement('tr');
    $cost->setAttribute('role','cost');
    $table->appendChild($cost);

    my $author = $doc->createElement('td');
    my $text = XML::LibXML::Text->new($module);
    $author->appendChild($text);
    $cost->appendChild($author);

# print per module cost
    foreach my $author (sort keys %authorscosts) {
	my $cell = $doc->createElement('td');
	$cell->setAttribute('role',$author);
	$cell->setAttribute('align','right');
	my $sum = 0;
	$sum =sprintf("%.0f",$authorsmodulescosts{$module.$author}/10*$price) if ($authorsmodulescosts{$module.$author});
	my $text = XML::LibXML::Text->new($sum);
	$cell->appendChild($text);
	$cost->appendChild($cell);
#	print "$module $author:\t$sum:\t$modulescosts{$module.$author}/10*$price\n";
#    print "$modulefile\n";
    }

#    print "$author\t$sum\n";
}

# print table footer
my $totalline = $doc->createElement('tr');
$table->appendChild($totalline);
$cell = $doc->createElement('td');
$text = XML::LibXML::Text->new("TOTAL");
$cell->appendChild($text);
$totalline->appendChild($cell);

# print per author total cost
foreach my $author (sort keys %authorscosts) {
    my $sum=sprintf("%.0f",$authorscosts{$author}/10*$price); 
    my $cell = $doc->createElement('td');
    $cell->setAttribute('align','right');
    my $text = XML::LibXML::Text->new($sum);
    $cell->appendChild($text);
    $totalline->appendChild($cell);
#    print "$modulefile\n";
    }


print $doc->toString;
